# Generated by Django 4.0 on 2021-12-28 10:20

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='DingDingMessage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=64, null=True, verbose_name='Title')),
                ('content', models.TextField(verbose_name='Content')),
                ('at_mobiles', models.JSONField(blank=True, null=True, verbose_name='At Mobiles')),
                ('msgtype', models.CharField(choices=[('text', 'Text'), ('link', 'Link'), ('markdown', 'Markdown'), ('actionCard', 'ActionCard')], default='markdown', max_length=32, verbose_name='Message Type')),
                ('is_at_all', models.BooleanField(default=False, verbose_name='At All')),
                ('extra', models.JSONField(blank=True, null=True, verbose_name='API Kwargs')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
            ],
            options={
                'verbose_name': 'DingDing Message',
                'db_table': 'dingding_message',
            },
        ),
        migrations.CreateModel(
            name='EmailMessage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=128, verbose_name='Subject')),
                ('content', models.TextField(blank=True, null=True, verbose_name='Content')),
                ('sender', models.EmailField(default='webmaster@localhost', max_length=254, verbose_name='Sender')),
                ('receivers', models.JSONField(verbose_name='Receivers')),
                ('cc', models.JSONField(blank=True, null=True, verbose_name='CC')),
                ('content_subtype', models.CharField(default='text', max_length=32, verbose_name='Content Subtype')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
            ],
            options={
                'verbose_name': 'Email Message',
                'db_table': 'email_message',
            },
        ),
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, verbose_name='Name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('code', models.CharField(max_length=4, unique=True, verbose_name='Template Code')),
                ('title', models.CharField(blank=True, max_length=64, null=True, verbose_name='Title')),
                ('text', models.TextField(verbose_name='Template Text')),
                ('kwargs', models.JSONField(blank=True, null=True, verbose_name='Kwargs')),
            ],
            options={
                'verbose_name': 'Template',
                'db_table': 'template',
            },
        ),
        migrations.CreateModel(
            name='WebsocketMessage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=64, null=True, verbose_name='Title')),
                ('content', models.TextField(blank=True, null=True, verbose_name='Content')),
                ('groups', models.JSONField(verbose_name='Receive Groups')),
                ('msgtype', models.CharField(default='notify', max_length=32, verbose_name='Message Type')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
            ],
            options={
                'verbose_name': 'Websocket Message',
                'db_table': 'websocket_message',
            },
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('has_read', models.BooleanField(default=False, verbose_name='Read Or Not')),
                ('push_state', models.CharField(choices=[('ready', 'Ready'), ('sucess', 'Sucess'), ('failed', 'Failed')], default='ready', max_length=16)),
                ('etag', models.TextField(blank=True, null=True, verbose_name='Etag')),
                ('object_id', models.PositiveIntegerField()),
                ('notified_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('content_type', models.ForeignKey(limit_choices_to={'model__in': ('DingDingMessage', 'WebsocketMessage', 'EmailMessage')}, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Message Type')),
                ('to', models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name='Receivers')),
            ],
            options={
                'verbose_name': 'Notification',
                'db_table': 'notification',
            },
        ),
    ]



from channels.generic.websocket import AsyncJsonWebsocketConsumer


class NotificationConsumer(AsyncJsonWebsocketConsumer):
    """
    This notify consumer handles websocket connections for single user client.
    It uses AsyncJsonWebsocketConsumer, which means all the handling functions
    must be async functions, and any sync work (like ORM access) has to be
    behind database_sync_to_async or sync_to_async. For more, read
    http://channels.readthedocs.io/en/latest/topics/consumers.html
    """
    def get_group_name(self):
        return f'notify-{self.scope["user"].pk}'

    async def connect(self):
        await self.channel_layer.group_add(self.get_group_name(), self.channel_name)
        await self.accept()

    async def disconnect(self, close_code):
        await self.channel_layer.group_discard(self.get_group_name(), self.channel_name)

    async def notify_message(self, event):
        """
        Send message to WebSocket
        """
        await self.send_json({
            'msgtype': event['msgtype'],
            'title': event['title'],
            'message': event['message'],
        })



import requests
import warnings
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
from django.db.transaction import atomic
from asgiref.sync import async_to_sync
import channels.layers
import logging
from .models import (
    DingDingMessage,
    Notification,
    EmailMessage,
    WebsocketMessage,
    Template,
)
from django.utils.module_loading import import_string

logger = logging.getLogger(__name__)


DEFAULT_COALESCE_TIME = 60


class NotificationBackend:
    notification_class = Notification
    template_class = Template
    message_class = None
    default_title = "Notification"

    # def __init__(self, *args, **kwargs):
    #     try:
    #         self.notification_settting = settings.DJANGO_NOTIFICATION
    #     except AttributeError:
    #         raise ImproperlyConfigured(f"'DJANGO_NOTIFICATION' must be set in settings.py")
    #
    #     self.notification_settting =

    def process_notification(self, instance, **kwargs):
        ...

    def process_message(self, message, receivers, title, **kwargs):
        ...

    def on_failure(self, message, notification, exc):
        notification.push_state = notification.FAILED
        notification.save(update_fields=("push_state",))

    def on_success(self, message, notification):
        notification.push_state = notification.SUCCESS
        notification.save(update_fields=("push_state",))

    @staticmethod
    def get_valid_kwargs(model, kwargs):
        if kwargs is None:
            return {}

        fields = [f.name for f in model._meta.get_fields()]
        return {k: v for k, v in kwargs.items() if k in fields}

    @atomic
    def perform_save(self, content, receivers, title, message_kwargs, notify_kwargs):
        """
        Process message
        """
        message = self.message_class(
            content=content, **self.get_valid_kwargs(self.message_class, message_kwargs)
        )
        self.process_message(message, receivers, title, **(message_kwargs or {}))
        message.save()
        notification = self.notification_class(
            content_object=message,
            **self.get_valid_kwargs(self.notification_class, notify_kwargs),
        )
        self.process_notification(notification, **(notify_kwargs or {}))
        notification.save()
        notification.to.set(receivers)
        return message, notification

    def send_with_template(self, receivers, context, template, title=None, **kwargs):
        content = template.render(context)
        return self.send(receivers, content, title=title, **kwargs)

    def send_message(self, instance, **kwargs):
        raise NotImplementedError

    def send(
        self,
        receivers,
        content,
        title=None,
        allow_resend=False,
        message_kwargs=None,
        notify_kwargs=None,
        **kwargs,
    ):
        """
        For details, see: https://ding-doc.dingtalk.com/doc#/serverapi2/qf2nxq
        """
        etag = notify_kwargs and notify_kwargs.get("etag")
        if etag and not allow_resend:
            if self.notification_class.objects.filter(etag=etag).exists():
                logger.info("重复消息！")
                return

        instance, notification = self.perform_save(
            content,
            receivers,
            title,
            message_kwargs=message_kwargs,
            notify_kwargs=notify_kwargs,
        )
        try:
            self.send_message(instance, **kwargs)
        except Exception as exc:
            self.on_failure(instance, notification, exc)
        else:
            self.on_success(instance, notification)

    def list(self):
        """
        Return notication queryset
        """
        return self.notification_class.objects.all()

    def clear(self):
        """
        Clear notification
        """
        self.notification_class.objects.delete()
        self.message_class.objects.delete()


class DingDingBotNotificationBackend(NotificationBackend):
    """
    A backend handle dingding message.
    """

    default_phone_field = "phone"
    message_class = DingDingMessage

    def __init__(self, webhook=None, user_phone_field=None, **kwargs):
        self.webhook = webhook or getattr(settings, "NOTIFICATION_DINGDING_WEBHOOK", None)
        if self.webhook is None:
            raise ImproperlyConfigured(f"'NOTIFICATION_DINGDING_WEBHOOK' must be set in settings.py")

        self.user_phone_field = user_phone_field or self.default_phone_field
        self.session = requests.Session()

    def process_message(self, message, receivers, title, **kwargs):
        try:
            message.title = title or self.default_title
            if not message.at_mobiles:
                message.at_mobiles = [
                    str(getattr(r, self.user_phone_field)) for r in receivers
                ]
        except AttributeError:
            pass

    def send_message(self, instance, **kwargs):
        """
        For details, see: https://ding-doc.dingtalk.com/doc#/serverapi2/qf2nxq
        """
        message = instance.to_message()
        resp = self.session.post(self.webhook, json=message)
        assert resp.json()["errcode"] == 0, resp.json()["errmsg"]


class EmailNotificationBackend(NotificationBackend):
    """
    A backend handle email message
    """

    message_class = EmailMessage

    def process_message(self, message, receivers, title, **kwargs):
        message.receivers = [r.email for r in receivers]
        message.subject = title or self.default_title

    def send_message(self, instance, **kwargs):
        """
        For details, see: ...
        """
        message = instance.to_message()
        message.send()


class WebsocketNotificationBackend(NotificationBackend):
    """
    A backend handle websocket message.
    """

    group_prefix = "notify"
    message_class = WebsocketMessage

    def get_group_name(self, user):
        return f"{self.group_prefix}-{user.pk}"

    def process_message(self, message, receivers, title, **kwargs):
        message.groups = [self.get_group_name(r) for r in receivers]
        message.title = title

    def send_message(self, instance, **kwargs):
        """
        For details, see: https://ding-doc.dingtalk.com/doc#/serverapi2/qf2nxq
        """
        message = instance.to_message()
        channel_layer = channels.layers.get_channel_layer()
        for group_name in instance.groups:
            async_to_sync(channel_layer.group_send)(group_name, message)


def notify(
    receivers,
    message=None,
    title=None,
    template_code=None,
    context=None,
    backends=None,
    allow_resend=True,
    save=False,
    message_coalesce=False,
    message_kwargs=None,
    notify_kwargs=None,
    **kwargs,
):
    try:
        backends = backends or settings.NOTIFICATION_BACKENDS
    except AttributeError:
        raise ImproperlyConfigured(
            "You must set NOTIFICATION_BACKENDS in 'settings.py'"
        )

    if message is None and not all([template_code, context]):
        raise ValueError("You must provide message or template.")

    if allow_resend is False:
        save = True

    for backend_cls in backends:
        if isinstance(backend_cls, str):
            backend_cls = import_string(backend_cls)

        if message:
            backend = backend_cls(**kwargs)
            backend.send(
                receivers,
                message,
                title=title,
                allow_resend=allow_resend,
                message_kwargs=message_kwargs,
                notify_kwargs=notify_kwargs,
                **kwargs,
            )
        else:
            try:
                template = backend_cls.template_class.objects.get(code=template_code)
            except ObjectDoesNotExist:
                raise ValueError(f"Template: {template_code} doesn't exist.")
            # `kwargs` is prior to `kwargs` defined in template
            title = title or template.title
            if template.kwargs and isinstance(template.kwargs, dict):
                default_kwargs = template.kwargs
                default_kwargs.update(kwargs)
            else:
                default_kwargs = kwargs

            backend = backend_cls(**default_kwargs)

            if message_coalesce and not isinstance(
                backend, DingDingBotNotificationBackend
            ):
                warnings.warn(
                    "`message_coalesce` is ignored, as it is now only supported by `DingDingBotNotificationBackend`.",
                    stacklevel=2,
                )
                message_coalesce = False

            backend.send_with_template(
                receivers,
                context,
                template,
                title=title,
                allow_resend=allow_resend,
                save=save,
                message_coalesce=message_coalesce,
                message_kwargs=message_kwargs,
                notify_kwargs=notify_kwargs,
                **default_kwargs,
            )


def notify_by_dingding(
    receivers,
    message=None,
    context=None,
    template_code=None,
    at_mobiles=None,
    msgtype="markdown",
    is_at_all=False,
    extra=None,
    allow_resend=True,
    save=False,
    message_coalesce=False,
    notify_kwargs=None,
    **kwargs,
):
    """
    Shortcut for dingding notification
    """
    backend_cls = DingDingBotNotificationBackend
    message_kwargs = dict(
        at_mobiles=at_mobiles, msgtype=msgtype, is_at_all=is_at_all, extra=extra
    )
    return notify(
        receivers,
        message=message,
        context=context,
        template_code=template_code,
        backends=(backend_cls,),
        allow_resend=allow_resend,
        save=save,
        message_coalesce=message_coalesce,
        message_kwargs=message_kwargs,
        notify_kwargs=notify_kwargs,
        **kwargs,
    )


def notify_by_email(
    receivers,
    message=None,
    context=None,
    template_code=None,
    cc=None,
    content_subtype="text",
    allow_resend=True,
    save=False,
    notify_kwargs=None,
    **kwargs,
):
    """
    Shortcut for email notification
    """
    backend_cls = EmailNotificationBackend
    message_kwargs = dict(cc=cc, content_subtype=content_subtype)
    return notify(
        receivers,
        message=message,
        context=context,
        template_code=template_code,
        backends=(backend_cls,),
        allow_resend=allow_resend,
        save=save,
        message_kwargs=message_kwargs,
        notify_kwargs=notify_kwargs,
        **kwargs,
    )


def notify_by_websocket(
    receivers,
    message=None,
    context=None,
    template_code=None,
    msgtype=None,
    allow_resend=True,
    save=False,
    notify_kwargs=None,
    **kwargs,
):
    """
    Shortcut for websocket notification
    """
    backend_cls = WebsocketNotificationBackend
    message_kwargs = {}
    if msgtype:
        message_kwargs.update(msgtype=msgtype)

    return notify(
        receivers,
        message=message,
        context=context,
        template_code=template_code,
        backends=(backend_cls,),
        allow_resend=allow_resend,
        save=save,
        message_kwargs=message_kwargs,
        notify_kwargs=notify_kwargs,
        **kwargs,
    )



from base64 import b64encode
from decorator import wraps


def make_etag(*args):
    """
    Generate custom etag
    """
    return "-".join(b64encode(tag.encode("utf-8")).decode("utf-8") for tag in args)




from django.contrib import admin
from . import models

from .models import Notification
from django.contrib.contenttypes.admin import GenericTabularInline

# Register your models here.


class NotificationAdmin(GenericTabularInline):
    model = models.Notification
    extra = 0

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.prefetch_related("to")


@admin.register(models.Template)
class TemplateAdmin(admin.ModelAdmin):
    list_display = ("name", "description", "code", "title", "text", "kwargs")
    list_filter = ("name", "code")
    ordering = ("name",)


@admin.register(models.DingDingMessage)
class DingDingMessageAdmin(admin.ModelAdmin):
    list_display = [
        "title",
        "content",
        "at_mobiles",
        "is_at_all",
        "extra",
        "created_at",
    ]
    list_filter = ("title", "created_at")
    inlines = (NotificationAdmin,)
    ordering = ("-id",)

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.prefetch_related("notify")


@admin.register(models.EmailMessage)
class EmailMessageAdmin(admin.ModelAdmin):
    list_display = [
        "subject",
        "sender",
        "receivers",
        "cc",
        "content_subtype",
        "content",
        "created_at",
    ]
    list_filter = ("subject", "content_subtype", "created_at")
    inlines = (NotificationAdmin,)
    ordering = ("-id",)

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.prefetch_related("notify")


@admin.register(models.WebsocketMessage)
class WebsocketMessageAdmin(admin.ModelAdmin):
    list_display = ["title", "content", "msgtype", "groups", "created_at"]
    inlines = (NotificationAdmin,)
    ordering = ("-id",)

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.prefetch_related("notify")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['notification', 'notification.migrations']

package_data = \
{'': ['*']}

install_requires = \
['channels>=3.0.4',
 'django-filter>=21.1',
 'django>=3.2',
 'djangorestframework>=3.13.1',
 'uvicorn>=0.16.0']

setup_kwargs = {
    'name': 'django-user-notification',
    'version': '0.4.3',
    'description': 'Django message notification package',
    'long_description': '# django-user-notification\n',
    'author': 'Aiden Lu',
    'author_email': 'allaher@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/aiden520/django-user-notification',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)



from django.conf import settings
from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericRelation
from django.contrib.contenttypes import fields
from django.utils.translation import gettext_lazy as _
from django.core.mail import EmailMessage as DjangoEmailMessage

# Create your models here.


class Notification(models.Model):
    READY = 0
    SUCCESS = 1
    FAILED = 2

    PUSH_STATE_CHOICE = (
        (READY, _('Pending')),
        (SUCCESS, _('Success')),
        (FAILED, _('Failure')),
    )

    has_read = models.BooleanField(verbose_name=_('Read Or Not'), default=False)
    push_state = models.PositiveIntegerField(choices=PUSH_STATE_CHOICE, default=READY)
    to = models.ManyToManyField(settings.AUTH_USER_MODEL, verbose_name=_('Receivers'))
    etag = models.TextField(verbose_name=_('Etag'), null=True, blank=True)
    content_type = models.ForeignKey(
        ContentType,
        verbose_name=_("Message Type"),
        on_delete=models.CASCADE,
        limit_choices_to={"model__in": ("DingDingMessage", "WebsocketMessage", "EmailMessage")},
    )
    object_id = models.PositiveIntegerField()
    content_object = fields.GenericForeignKey()
    notified_at = models.DateTimeField(auto_now_add=True, verbose_name=_('Created At'))
    updated_at = models.DateTimeField(auto_now=True, verbose_name=_('Updated At'))

    class Meta:
        verbose_name = _("Notification")
        db_table = 'notification'


class Template(models.Model):
    """
    Message template
    """
    name = models.CharField(max_length=64, verbose_name=_("Name"))
    description = models.TextField(verbose_name=_("Description"), null=True, blank=True)
    code = models.CharField(max_length=4, verbose_name=_("Template Code"), unique=True)
    title = models.CharField(max_length=64, verbose_name=_("Title"), null=True, blank=True)
    text = models.TextField(verbose_name=_("Template Text"))
    kwargs = models.JSONField(verbose_name=_("Kwargs"), null=True, blank=True)

    def __str__(self):
        return self.text

    def render(self, context):
        """
        Render message
        """
        try:
            return self.text.format(**context)
        except Exception:
            raise ValueError("Render message failed!")

    class Meta:
        verbose_name = _("Template")
        db_table = 'template'


class DingDingMessage(models.Model):
    """
    DingDing Message
    """
    TEXT = 'text'
    LINK = 'link'
    MARKDOWN = 'markdown'
    ACTION_CARD = 'actionCard'
    MESSAGE_TYPES = (
        (TEXT, "Text"),
        (LINK, "Link"),
        (MARKDOWN, "Markdown"),
        (ACTION_CARD, "ActionCard"),
    )

    title = models.CharField(max_length=64, null=True, blank=True, verbose_name=_('Title'))
    content = models.TextField(verbose_name=_("Content"))
    at_mobiles = models.JSONField(verbose_name=_('At Mobiles'), null=True, blank=True)
    msgtype = models.CharField(max_length=32, choices=MESSAGE_TYPES, default=MARKDOWN, verbose_name=_('Message Type'))
    is_at_all = models.BooleanField(default=False, verbose_name=_('At All'))
    extra = models.JSONField(verbose_name=_("API Kwargs"), null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_('Created At'))
    notify = GenericRelation(Notification)

    def to_message(self):
        content_key = 'content' if self.msgtype == self.TEXT else 'text'
        # TODO change json format depending on msgtype.
        if self.at_mobiles:
            format_at_str = ("@" + mobile for mobile in self.at_mobiles)
            content = ' '.join(format_at_str) + '\n\n' + self.content
        else:
            content = self.content
        return {
            "msgtype": self.msgtype,
            "at": {"atMobiles": self.at_mobiles, "isAtAll": self.is_at_all},
            self.msgtype: {"title": self.title, content_key: content, **(self.extra or {})},
        }

    class Meta:
        verbose_name = _("DingDing Message")
        db_table = 'dingding_message'


class WebsocketMessage(models.Model):
    """
    Websocket Message
    """
    DEFAULT_MESSAGE_TYPE = 'notify'

    title = models.CharField(max_length=64, null=True, blank=True, verbose_name=_('Title'))
    content = models.TextField(verbose_name=_("Content"), null=True, blank=True)
    groups = models.JSONField(verbose_name=_('Receive Groups'))
    msgtype = models.CharField(max_length=32, verbose_name=_('Message Type'), default=DEFAULT_MESSAGE_TYPE)
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_('Created At'))
    notify = GenericRelation(Notification)

    def to_message(self):
        return {
            "type": 'notify.message',
            "msgtype": self.msgtype,
            "title": self.title,
            "message": self.content,
        }

    class Meta:
        verbose_name = _("Websocket Message")
        db_table = 'websocket_message'


class EmailMessage(models.Model):
    """
    Email Message
    """
    subject = models.CharField(max_length=128, verbose_name=_("Subject"))
    content = models.TextField(verbose_name=_("Content"), null=True, blank=True)
    sender = models.EmailField(verbose_name=_("Sender"), default=settings.DEFAULT_FROM_EMAIL)
    receivers = models.JSONField(verbose_name=_('Receivers'))
    cc = models.JSONField(null=True, blank=True, verbose_name=_("CC"))
    content_subtype = models.CharField(max_length=32, verbose_name=_("Content Subtype"), default="text")
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_('Created At'))
    notify = GenericRelation(Notification)

    def to_message(self):
        email = DjangoEmailMessage(self.subject, self.content, to=self.receivers, cc=self.cc)
        email.content_subtype = self.content_subtype
        return email

    class Meta:
        verbose_name = _("Email Message")
        db_table = 'email_message'
